require "spec_helper"

describe OneEye::Base do
  it "defines defaults", :defaults do
    expect(OneEye::User.find(41644529).admin_rights["institution"]).to eq "schools/404013"
    expect(OneEye::User.new.admin_rights).to eq({})
  end

  it "finds by id", :find do
    expect(OneEye::District.find(22037).id).to eq(22037)
    expect(OneEye::School.find(334517).id).to eq(334517)
    expect(OneEye::User.find(34954995).id).to eq(34954995)
    expect(OneEye::Group.find(110).id).to eq(110)
  end

  it "finds with a hash of options", :find do
    expect(OneEye::District.find(:id => 22037).id).to eq(22037)
  end

  it "finds a collection of models using where", :where do
    expect(OneEye::District.where.first.id).to eq(1)
    expect(OneEye::School.where.first.id).to eq(1)
  end

  it "uses params to query with where", :where do
    @schools = OneEye::School.where(:district_id => 22037)
    expect(@schools.first.abbreviated_name).to eq("OVOXO Academy")
  end

  it "appropriately sets query key to where", :where do
    @schools = OneEye::School.where(:district_id => 22037, :query => "october")
    expect(@schools.first.name).to eq("October's Very Own Academy")
  end

  it "replaces module names with URLs", :deep_module_nesting do
    expect(OneEye::Insights::SnapshotSuggestedGroup.find(55).school_id).to eq(406879)
  end

  it "dynamically dispatches custom finders", :finders do
    expect(OneEye::Insights::SnapshotUsage.districts(:district_id => 22037).first.num_teachers).to eq(5)
    expect(OneEye::Insights::SnapshotPerformance.schools(:district_id => 22037,
                                                         :assessment_area => "CCSS.ELA")
                                                         .first.progress_percent).to eq(4.73)
  end

  it "sets attributes", :attributes do
    @school = OneEye::School.new :city => "Philly", :state => "PA"
    expect(@school.attributes).to eq({"city" => "Philly", "state" => "PA"})
  end

  describe "Creating and destroying" do
    before(:each) do
      @district = OneEye::District.create(:name => "YMCMB District")
    end

    it "creates instance via a backend post", :destroy do
      expect(@district.name).to eq("YMCMB District")
      @district.destroy
    end
  end

  it "updates instances", :create do
    @district = OneEye::District.new(:id => 22037)
    expect(@district.update(:motto => "Get Money Get Paid").motto).to eq("Get Money Get Paid")

    @school = OneEye::School.new(:id => 334517)
    expect(@school.update(:motto => "Get Money Get Paid").motto).to eq("Get Money Get Paid")

    @user = OneEye::User.new(:id => 34954995)
    expect(@user.update(:verified_institution_member => true).verified_institution_member).to eq(true)
  end

  it "handles error codes", :error_codes do
    module OneEye
      class Base
        def self.api
          OneEye::API.new "badAPIToken"
        end
      end
    end

    @user = OneEye::User.find(1)
    @user.update :fun => true
    expect(@user.status_code).to eq "404"

    module OneEye
      class Base
        def self.api
          OneEye::API.new
        end
      end
    end
  end

end
